/*
Copyright 2021 Rancher Labs, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	v1 "github.com/rancher/rancher/pkg/apis/rke.cattle.io/v1"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"github.com/rancher/wrangler/pkg/kv"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type RKEClusterHandler func(string, *v1.RKECluster) (*v1.RKECluster, error)

type RKEClusterController interface {
	generic.ControllerMeta
	RKEClusterClient

	OnChange(ctx context.Context, name string, sync RKEClusterHandler)
	OnRemove(ctx context.Context, name string, sync RKEClusterHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() RKEClusterCache
}

type RKEClusterClient interface {
	Create(*v1.RKECluster) (*v1.RKECluster, error)
	Update(*v1.RKECluster) (*v1.RKECluster, error)
	UpdateStatus(*v1.RKECluster) (*v1.RKECluster, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.RKECluster, error)
	List(namespace string, opts metav1.ListOptions) (*v1.RKEClusterList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.RKECluster, err error)
}

type RKEClusterCache interface {
	Get(namespace, name string) (*v1.RKECluster, error)
	List(namespace string, selector labels.Selector) ([]*v1.RKECluster, error)

	AddIndexer(indexName string, indexer RKEClusterIndexer)
	GetByIndex(indexName, key string) ([]*v1.RKECluster, error)
}

type RKEClusterIndexer func(obj *v1.RKECluster) ([]string, error)

type rKEClusterController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewRKEClusterController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) RKEClusterController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &rKEClusterController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromRKEClusterHandlerToHandler(sync RKEClusterHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.RKECluster
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.RKECluster))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *rKEClusterController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.RKECluster))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateRKEClusterDeepCopyOnChange(client RKEClusterClient, obj *v1.RKECluster, handler func(obj *v1.RKECluster) (*v1.RKECluster, error)) (*v1.RKECluster, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *rKEClusterController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *rKEClusterController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *rKEClusterController) OnChange(ctx context.Context, name string, sync RKEClusterHandler) {
	c.AddGenericHandler(ctx, name, FromRKEClusterHandlerToHandler(sync))
}

func (c *rKEClusterController) OnRemove(ctx context.Context, name string, sync RKEClusterHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromRKEClusterHandlerToHandler(sync)))
}

func (c *rKEClusterController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *rKEClusterController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *rKEClusterController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *rKEClusterController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *rKEClusterController) Cache() RKEClusterCache {
	return &rKEClusterCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *rKEClusterController) Create(obj *v1.RKECluster) (*v1.RKECluster, error) {
	result := &v1.RKECluster{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *rKEClusterController) Update(obj *v1.RKECluster) (*v1.RKECluster, error) {
	result := &v1.RKECluster{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *rKEClusterController) UpdateStatus(obj *v1.RKECluster) (*v1.RKECluster, error) {
	result := &v1.RKECluster{}
	return result, c.client.UpdateStatus(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *rKEClusterController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *rKEClusterController) Get(namespace, name string, options metav1.GetOptions) (*v1.RKECluster, error) {
	result := &v1.RKECluster{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *rKEClusterController) List(namespace string, opts metav1.ListOptions) (*v1.RKEClusterList, error) {
	result := &v1.RKEClusterList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *rKEClusterController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *rKEClusterController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1.RKECluster, error) {
	result := &v1.RKECluster{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type rKEClusterCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *rKEClusterCache) Get(namespace, name string) (*v1.RKECluster, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1.RKECluster), nil
}

func (c *rKEClusterCache) List(namespace string, selector labels.Selector) (ret []*v1.RKECluster, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1.RKECluster))
	})

	return ret, err
}

func (c *rKEClusterCache) AddIndexer(indexName string, indexer RKEClusterIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.RKECluster))
		},
	}))
}

func (c *rKEClusterCache) GetByIndex(indexName, key string) (result []*v1.RKECluster, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.RKECluster, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.RKECluster))
	}
	return result, nil
}

type RKEClusterStatusHandler func(obj *v1.RKECluster, status v1.RKEClusterStatus) (v1.RKEClusterStatus, error)

type RKEClusterGeneratingHandler func(obj *v1.RKECluster, status v1.RKEClusterStatus) ([]runtime.Object, v1.RKEClusterStatus, error)

func RegisterRKEClusterStatusHandler(ctx context.Context, controller RKEClusterController, condition condition.Cond, name string, handler RKEClusterStatusHandler) {
	statusHandler := &rKEClusterStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromRKEClusterHandlerToHandler(statusHandler.sync))
}

func RegisterRKEClusterGeneratingHandler(ctx context.Context, controller RKEClusterController, apply apply.Apply,
	condition condition.Cond, name string, handler RKEClusterGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &rKEClusterGeneratingHandler{
		RKEClusterGeneratingHandler: handler,
		apply:                       apply,
		name:                        name,
		gvk:                         controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	controller.OnChange(ctx, name, statusHandler.Remove)
	RegisterRKEClusterStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type rKEClusterStatusHandler struct {
	client    RKEClusterClient
	condition condition.Cond
	handler   RKEClusterStatusHandler
}

func (a *rKEClusterStatusHandler) sync(key string, obj *v1.RKECluster) (*v1.RKECluster, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status.DeepCopy()
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(&newStatus, "", nil)
		} else {
			a.condition.SetError(&newStatus, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, &newStatus) {
		if a.condition != "" {
			// Since status has changed, update the lastUpdatedTime
			a.condition.LastUpdated(&newStatus, time.Now().UTC().Format(time.RFC3339))
		}

		var newErr error
		obj.Status = newStatus
		newObj, newErr := a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
		if newErr == nil {
			obj = newObj
		}
	}
	return obj, err
}

type rKEClusterGeneratingHandler struct {
	RKEClusterGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *rKEClusterGeneratingHandler) Remove(key string, obj *v1.RKECluster) (*v1.RKECluster, error) {
	if obj != nil {
		return obj, nil
	}

	obj = &v1.RKECluster{}
	obj.Namespace, obj.Name = kv.RSplit(key, "/")
	obj.SetGroupVersionKind(a.gvk)

	return nil, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects()
}

func (a *rKEClusterGeneratingHandler) Handle(obj *v1.RKECluster, status v1.RKEClusterStatus) (v1.RKEClusterStatus, error) {
	objs, newStatus, err := a.RKEClusterGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	return newStatus, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
