/*
Copyright 2021 Rancher Labs, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	v1 "github.com/rancher/rancher/pkg/apis/rke.cattle.io/v1"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"github.com/rancher/wrangler/pkg/kv"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type UnmanagedMachineHandler func(string, *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error)

type UnmanagedMachineController interface {
	generic.ControllerMeta
	UnmanagedMachineClient

	OnChange(ctx context.Context, name string, sync UnmanagedMachineHandler)
	OnRemove(ctx context.Context, name string, sync UnmanagedMachineHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() UnmanagedMachineCache
}

type UnmanagedMachineClient interface {
	Create(*v1.UnmanagedMachine) (*v1.UnmanagedMachine, error)
	Update(*v1.UnmanagedMachine) (*v1.UnmanagedMachine, error)
	UpdateStatus(*v1.UnmanagedMachine) (*v1.UnmanagedMachine, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.UnmanagedMachine, error)
	List(namespace string, opts metav1.ListOptions) (*v1.UnmanagedMachineList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.UnmanagedMachine, err error)
}

type UnmanagedMachineCache interface {
	Get(namespace, name string) (*v1.UnmanagedMachine, error)
	List(namespace string, selector labels.Selector) ([]*v1.UnmanagedMachine, error)

	AddIndexer(indexName string, indexer UnmanagedMachineIndexer)
	GetByIndex(indexName, key string) ([]*v1.UnmanagedMachine, error)
}

type UnmanagedMachineIndexer func(obj *v1.UnmanagedMachine) ([]string, error)

type unmanagedMachineController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewUnmanagedMachineController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) UnmanagedMachineController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &unmanagedMachineController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromUnmanagedMachineHandlerToHandler(sync UnmanagedMachineHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.UnmanagedMachine
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.UnmanagedMachine))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *unmanagedMachineController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.UnmanagedMachine))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateUnmanagedMachineDeepCopyOnChange(client UnmanagedMachineClient, obj *v1.UnmanagedMachine, handler func(obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error)) (*v1.UnmanagedMachine, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *unmanagedMachineController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *unmanagedMachineController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *unmanagedMachineController) OnChange(ctx context.Context, name string, sync UnmanagedMachineHandler) {
	c.AddGenericHandler(ctx, name, FromUnmanagedMachineHandlerToHandler(sync))
}

func (c *unmanagedMachineController) OnRemove(ctx context.Context, name string, sync UnmanagedMachineHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromUnmanagedMachineHandlerToHandler(sync)))
}

func (c *unmanagedMachineController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *unmanagedMachineController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *unmanagedMachineController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *unmanagedMachineController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *unmanagedMachineController) Cache() UnmanagedMachineCache {
	return &unmanagedMachineCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *unmanagedMachineController) Create(obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error) {
	result := &v1.UnmanagedMachine{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *unmanagedMachineController) Update(obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error) {
	result := &v1.UnmanagedMachine{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *unmanagedMachineController) UpdateStatus(obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error) {
	result := &v1.UnmanagedMachine{}
	return result, c.client.UpdateStatus(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *unmanagedMachineController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *unmanagedMachineController) Get(namespace, name string, options metav1.GetOptions) (*v1.UnmanagedMachine, error) {
	result := &v1.UnmanagedMachine{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *unmanagedMachineController) List(namespace string, opts metav1.ListOptions) (*v1.UnmanagedMachineList, error) {
	result := &v1.UnmanagedMachineList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *unmanagedMachineController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *unmanagedMachineController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1.UnmanagedMachine, error) {
	result := &v1.UnmanagedMachine{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type unmanagedMachineCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *unmanagedMachineCache) Get(namespace, name string) (*v1.UnmanagedMachine, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1.UnmanagedMachine), nil
}

func (c *unmanagedMachineCache) List(namespace string, selector labels.Selector) (ret []*v1.UnmanagedMachine, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1.UnmanagedMachine))
	})

	return ret, err
}

func (c *unmanagedMachineCache) AddIndexer(indexName string, indexer UnmanagedMachineIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.UnmanagedMachine))
		},
	}))
}

func (c *unmanagedMachineCache) GetByIndex(indexName, key string) (result []*v1.UnmanagedMachine, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.UnmanagedMachine, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.UnmanagedMachine))
	}
	return result, nil
}

type UnmanagedMachineStatusHandler func(obj *v1.UnmanagedMachine, status v1.UnmanagedMachineStatus) (v1.UnmanagedMachineStatus, error)

type UnmanagedMachineGeneratingHandler func(obj *v1.UnmanagedMachine, status v1.UnmanagedMachineStatus) ([]runtime.Object, v1.UnmanagedMachineStatus, error)

func RegisterUnmanagedMachineStatusHandler(ctx context.Context, controller UnmanagedMachineController, condition condition.Cond, name string, handler UnmanagedMachineStatusHandler) {
	statusHandler := &unmanagedMachineStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromUnmanagedMachineHandlerToHandler(statusHandler.sync))
}

func RegisterUnmanagedMachineGeneratingHandler(ctx context.Context, controller UnmanagedMachineController, apply apply.Apply,
	condition condition.Cond, name string, handler UnmanagedMachineGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &unmanagedMachineGeneratingHandler{
		UnmanagedMachineGeneratingHandler: handler,
		apply:                             apply,
		name:                              name,
		gvk:                               controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	controller.OnChange(ctx, name, statusHandler.Remove)
	RegisterUnmanagedMachineStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type unmanagedMachineStatusHandler struct {
	client    UnmanagedMachineClient
	condition condition.Cond
	handler   UnmanagedMachineStatusHandler
}

func (a *unmanagedMachineStatusHandler) sync(key string, obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status.DeepCopy()
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(&newStatus, "", nil)
		} else {
			a.condition.SetError(&newStatus, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, &newStatus) {
		if a.condition != "" {
			// Since status has changed, update the lastUpdatedTime
			a.condition.LastUpdated(&newStatus, time.Now().UTC().Format(time.RFC3339))
		}

		var newErr error
		obj.Status = newStatus
		newObj, newErr := a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
		if newErr == nil {
			obj = newObj
		}
	}
	return obj, err
}

type unmanagedMachineGeneratingHandler struct {
	UnmanagedMachineGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *unmanagedMachineGeneratingHandler) Remove(key string, obj *v1.UnmanagedMachine) (*v1.UnmanagedMachine, error) {
	if obj != nil {
		return obj, nil
	}

	obj = &v1.UnmanagedMachine{}
	obj.Namespace, obj.Name = kv.RSplit(key, "/")
	obj.SetGroupVersionKind(a.gvk)

	return nil, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects()
}

func (a *unmanagedMachineGeneratingHandler) Handle(obj *v1.UnmanagedMachine, status v1.UnmanagedMachineStatus) (v1.UnmanagedMachineStatus, error) {
	objs, newStatus, err := a.UnmanagedMachineGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	return newStatus, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
